# -*- coding: utf8 -*-
from __future__ import unicode_literals

import base64
import logging
import time

from driverng import Worklist
from driverng.astm import AstmMessageV2, HeaderSegment
from driverng.astm import HprimDate as HL7Date
from driverng.astm import HprimForeign as HL7Foreign
from driverng.astm import HprimString as HL7String
from driverng.astm import NoRangSegment, Segment
from driverng.message import BinGraphContainer

logger = logging.getLogger("driverng.yumizen_p8000")


class YumizenP8000GraphContainer(BinGraphContainer):
    def __init__(self, data=None):
        super(YumizenP8000GraphContainer, self).__init__()
        self["data"] = data
        self["format"] = "png"
        self["graph_type"] = "binary"


class YumizenP8000SegmentMSH(HeaderSegment):
    sending_application = HL7String(3)
    sending_facility = HL7String(4)
    receiving_application = HL7String(5)
    receiving_facility = HL7String(6)
    message_date = HL7Date(7)
    message_type = HL7String((9, 1), default="OML")
    message_code = HL7String((9, 2), default="O33")
    trigger_event = HL7String((9, 3), default="OML_O33")
    control_id = HL7String(10)
    processing_id = HL7String(11, default="P")
    version_id = HL7String(12, default="2.5")

    patients = HL7Foreign("PID", "header")
    orders = HL7Foreign("OBR", "header")


class YumizenP8000SegmentPID(Segment):
    patient_id = HL7String((4, 1))
    assigning_authority = HL7String((4, 4), default="LIS")
    identifier_type_code = HL7String((4, 5), default="PI")
    last_name = HL7String((6, 1))
    first_name = HL7String((6, 2))
    birthdate = HL7Date(8, fmt_date="%Y%m%d")
    sex = HL7String(9)
    address = HL7String((12, 1))
    city = HL7String((12, 3))
    zip_code = HL7String((12, 5))
    country = HL7String((12, 6))
    address_type = HL7String((12, 7))
    phone = HL7String(14)
    identity_unknow_indicator = HL7String(32, default="N")
    identity_reliability_code = HL7String(33, default="AL")

    visits = HL7Foreign("PV1", "patient")


class YumizenP8000SegmentPV1(Segment):
    petient_class = HL7String(3, default="N")
    admit_date = HL7Date(45)
    discharge_date = HL7Date(46)

    specimens = HL7Foreign("SPM", "visit")


class YumizenP8000SegmentSPM(Segment):
    sample_id = HL7String(3)
    specimen_type = HL7String(5)
    specimen_source_site = HL7String(9)
    collection_date = HL7Date(18)
    received_date = HL7Date(19)

    common_orders = HL7Foreign("ORC", "specimen")
    orders = HL7Foreign("OBR", "specimen")


class YumizenP8000SegmentORC(NoRangSegment):
    order_control = HL7String(2)
    placer_order_number = HL7String(3)
    filler_order_number = HL7String(4)
    placer_group_number = HL7String(5)
    transaction_date = HL7Date(10)
    enterer_location = HL7String(14)
    ordering_facility = HL7String((22, 1))
    ward_id = HL7String((22, 11))

    priorities = HL7Foreign("TQ1", "common_order")


class YumizenP8000SegmentTQ1(Segment):
    start_date = HL7Date(8)
    priority = HL7String(10)

    orders = HL7Foreign("OBR", "priority")
    results = HL7Foreign("OBX", "priority")


class YumizenP8000SegmentOBR(Segment):
    placer_order_number = HL7String(3)
    filler_order_number = HL7String(4)
    code = HL7String((5, 1))
    name = HL7String((5, 2))
    sending_application = HL7String((5, 3))
    observation_date = HL7Date(8)
    observation_end_date = HL7Date(9)
    specimen_action_code = HL7String(12)
    ordering_provider = HL7String(17)
    result_status = HL7String(25, default="P")
    specimen_source = HL7String((15, 7))

    common_orders = HL7Foreign("ORC", "order")
    results = HL7Foreign("OBX", "order")


class YumizenP8000SegmentOBX(Segment):
    value_type = HL7String(3)
    observation_identifier = HL7String((4, 1))
    observation_name = HL7String((4, 2))
    order_comment = HL7String((4, 3))
    observation_value = HL7String((6, 1))
    image_type = HL7String((6, 2))
    image_format = HL7String((6, 3))
    image_encoding = HL7String((6, 4))
    image = HL7String((6, 5))
    units = HL7String(7)
    result_status = HL7String(12)
    observation_date = HL7Date(15)
    equipment_identifier = HL7String(19)

    comments = HL7Foreign("NTE", "result")
    orders = HL7Foreign("OBR", "result")


class YumizenP8000SegmentNTE(Segment):
    comment = HL7String(4)


class Driver(AstmMessageV2):
    SPEC = {"code": "yumizen_p8000", "label": "Yumizen P8000"}
    SEGMENT_CLASS_MAP = {
        "MSH": YumizenP8000SegmentMSH,
        "PID": YumizenP8000SegmentPID,
        "PV1": YumizenP8000SegmentPV1,
        "SPM": YumizenP8000SegmentSPM,
        "ORC": YumizenP8000SegmentORC,
        "TQ1": YumizenP8000SegmentTQ1,
        "OBR": YumizenP8000SegmentOBR,
        "OBX": YumizenP8000SegmentOBX,
        "NTE": YumizenP8000SegmentNTE,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    OPTIONS = [
        AstmMessageV2.Option(
            "yumizen_p8000_sending_application", label="Code émetteur", default="MCA"
        ),
        AstmMessageV2.Option(
            "yumizen_p8000_sending_facility", label="Libellé émetteur", default="MCA"
        ),
        AstmMessageV2.Option(
            "yumizen_p8000_receiving_application",
            label="Code récepteur",
            default="YP8K",
        ),
        AstmMessageV2.Option(
            "yumizen_p8000_receiving_facility",
            label="Libellé récepteur",
            default="YP8K",
        ),
    ]

    def __init__(self, **kwargs):
        self.yumizen_p8000_sending_application = kwargs.pop(
            "yumizen_p8000_sending_application", "MCA"
        )
        self.yumizen_p8000_sending_facility = kwargs.pop(
            "yumizen_p8000_sending_facility", "MCA"
        )
        self.yumizen_p8000_receiving_application = kwargs.pop(
            "yumizen_p8000_receiving_application", "YP8K"
        )
        self.yumizen_p8000_receiving_facility = kwargs.pop(
            "yumizen_p8000_receiving_facility", "YP8K"
        )
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        return YumizenP8000SegmentMSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    def get_result(self, seg_obx, comments):
        result = {
            "code": seg_obx.observation_identifier,
            "value": seg_obx.observation_value,
            "unit": seg_obx.units,
            "date": seg_obx.observation_date,
            "instrument": seg_obx.equipment_identifier,
            "comments": comments,
        }
        return result

    def get_graph(self, seg_obx, comments):
        result = {
            "code": "GRAPHE_{}".format(seg_obx.observation_identifier),
            "value": "GRAPHE",
            "date": seg_obx.observation_date,
            "instrument": seg_obx.equipment_identifier,
            "comments": comments,
            "graph": YumizenP8000GraphContainer(data=base64.b64decode(seg_obx.image)),
        }
        return result

    def encode_worklist(self, worklist, context):
        message = self.create(
            sending_application=self.yumizen_p8000_sending_application,
            sending_facility=self.yumizen_p8000_sending_facility,
            receiving_application=self.yumizen_p8000_receiving_application,
            receiving_facility=self.yumizen_p8000_receiving_facility,
            message_date=context["now"],
            control_id=context.get("control_id", int(time.time() * 1000)),
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["priority", "rerun"],
            priority={"S": "S", worklist.ELSE: "R"},
        ):
            physician = [
                p for p in patient.physicians if p.type.lower() == "prescriber"
            ]
            seg_pid = message.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                address=patient.address,
                city=patient.city,
                zip_code=patient.postal_code,
                country=patient.country,
                phone=patient.phone,
            )
            seg_pv1 = seg_pid.visits.add()
            for sample, priority, rerun in patient.samples:
                seg_spm = seg_pv1.specimens.add(
                    sample_id=sample.sample_id,
                    specimen_type=sample.specimen,
                    collection_date=sample.collection_date,
                    received_date=sample.request_date,
                )
                for order in sample.orders:
                    seg_orc = seg_spm.common_orders.add(
                        order_control="RF" if rerun else "NW",
                        placer_order_number=sample.request_id,
                        filler_order_number=sample.request_id,
                        placer_group_number=sample.request_id,
                        transaction_date=sample.request_date,
                        enterer_location=sample.location,
                        ordering_facility=sample.location_name,
                        ward_id=sample.location,
                    )
                    seg_tq1 = seg_orc.priorities.add(
                        start_date=sample.request_date, priority=priority
                    )
                    seg_obr = seg_tq1.orders.add(
                        placer_order_number=sample.request_id,
                        filler_order_number=sample.request_id,
                        code=order.code,
                        name=order.label,
                        sending_application=self.yumizen_p8000_sending_application,
                        specimen_action_code=sample.specimen,
                        ordering_provider=physician[0].code if physician else "",
                    )
                    for result in sample.results:
                        seg_obr.results.add(
                            value_type=result.type,
                            observation_identifier=result.code,
                            observation_value=result.value,
                            unit=result.unit,
                            result_status=result.status,
                            observation_date=result.date,
                        )

        if message.patients:
            return message

    def decode_message(self, message):
        worklist = Worklist(context="result")
        if message.message_type == "OUL" and message.message_code == "R22":
            sample_id = ""
            for seg_pid in message.patients:
                for seg_pv1 in seg_pid.visits:
                    for seg_spm in seg_pv1.specimens:
                        sample_id = seg_spm.sample_id
                        sample = worklist.get_or_create(
                            sample_id=sample_id, type="patient"
                        )
                        for seg_obr in seg_spm.orders:
                            for seg_orc in seg_obr.common_orders:
                                for seg_tq1 in seg_orc.priorities:
                                    for seg_obx in seg_tq1.results:
                                        comments = []
                                        for seg_nte in seg_obx.comments:
                                            comments.append(seg_nte.comment)
                                        if seg_obx.value_type == "NM":
                                            sample.results.add(
                                                self.get_result(seg_obx, comments)
                                            )
                                        elif seg_obx.value_type == "ED":
                                            sample.results.add(
                                                self.get_graph(seg_obx, comments)
                                            )
                                        while seg_obx.orders:
                                            for seg_obr in seg_obx.orders:
                                                for seg_orc in seg_obr.common_orders:
                                                    for seg_tq1 in seg_orc.priorities:
                                                        for seg_obx in seg_tq1.results:
                                                            comments = []
                                                            for (
                                                                seg_nte
                                                            ) in seg_obx.comments:
                                                                comments.append(
                                                                    seg_nte.comment
                                                                )
                                                            if (
                                                                seg_obx.value_type
                                                                == "NM"
                                                            ):
                                                                sample.results.add(
                                                                    self.get_result(
                                                                        seg_obx,
                                                                        comments,
                                                                    )
                                                                )
                                                            elif (
                                                                seg_obx.value_type
                                                                == "ED"
                                                            ):
                                                                sample.results.add(
                                                                    self.get_graph(
                                                                        seg_obx,
                                                                        comments,
                                                                    )
                                                                )
        elif message.message_type == "ORU" and message.message_code == "R01":
            for seg_obr in message.orders:
                sample = worklist.get_or_create(
                    sample_id=seg_obr.filler_order_number,
                    control_lot=seg_obr.filler_order_number,
                    type="control",
                )
                for seg_obx in seg_obr.results:
                    if seg_obx.value_type == "NM":
                        sample.results.add(
                            code=seg_obx.observation_identifier,
                            value=seg_obx.observation_value,
                            unit=seg_obx.units,
                            date=seg_obx.observation_date,
                            instrument=seg_obx.equipment_identifier,
                        )
                    elif seg_obx.value_type == "ST":
                        sample.results.add(
                            code=seg_obx.observation_identifier,
                            value=seg_obx.observation_value,
                        )

        return worklist
